//****************************************************************** 
// PIC GEIGER COUNTER								G.SAMBLANCAT
//	
// Version 1.0 										25/5/2011
/*******************************************************************
// Version C18 2.40 du 17/11/2004
// Version MPLAB 8.40
/******************************************************************/
// Config C18:
// Optimization project : All sauf 'procedural'
/** I N C L U D E S ************************************************/

#include "p18f2455.h"
#include "iocfg.h"
#include "main.h"
#include "lcd.h"
#include "interrupt.h"

/**  V A R I A B L E S   G L O B A L E S ***************************/
#pragma	udata //section=gpr0

unsigned int pulses;
unsigned char top[LONG_AQU_TIME];
unsigned char ntop, n, tim50, plein;
unsigned char aqutime;
unsigned char affich;
unsigned char chrwait;				// char de la barre en bas ecran
unsigned char beepon;				// flag si buzzer actif

/** D E C L A R A T I O N S **************************************************/
#pragma code InterruptVectorLow = 0x08
void high_isr (void) {
	_asm
		goto isr
	_endasm
}

#pragma code InterruptVectorHigh = 0x18
void low_isr (void) {
	_asm
		goto lowisr
	_endasm
}

#pragma code page

/********************************************************************
 * Function:        void Initialize(void)
 * PreCondition:    None
 * Overview:        InitializeSystem is a centralize initialization routine.
 *******************************************************************/
void Initialize(void) {

 TRISA = 0b11111111;		// inputs
 TRISB = 0b00000011;		// LCD 4 bits datas,RS,E + INT0 + Bouton
 TRISC = 0b10111011;		// RC2->CCP1, RC6-> Buzzer

 LATA = 0xff;
 LATB = 0x00;
 LATC = 0x00;

 INTCON = 0x00;

 //Compteur time pour 1 sec
 tim50 = 1*20;
 ntop=0;

 //Allume afficheur
 TempoMS(99);

 //Init afficheur
 lcd_init();
 lcd_clear();

 //Init PWM & Timer 2 pour High Voltage (7.5kHz) 
 PR2 = 198;		// 128=11.6kHz	98=15.15kHz
 T2CON = 0b00010101;	// on, prescaler=1
 CCPR1L = 0x40;
 CCP1CON = 0b00001100;  // PWM mode CCP1

 //Init Timer 1 pour base de temps comptage
 TMR1H = 0x6E;
 TMR1L = 0;
 T1CON = 0b10110101;	// on,int clock, 16bits, 1:8 

 BUZZER = 1;
 TempoMS(50);
 BUZZER = 0;

 //Lance comptage ITs & RBPU disabled
 INTCON2 = 0b10000000;
 //Enable its INT & TMR0
 INTCON = 0b11110000;
 //Enable TMR1 it
 PIE1 = 0b00000001;

}//end Initialize

/*******************************************************************
 * Function:        void main(void)
 * Input:           None
 * Output:          None
 * Overview:        Main program entry point.
 ******************************************************************/
//Clock Fosc = 24MHz  (166nS par instruction)
void main(void) {
unsigned long tot;
float cpm, usv, nsv, ur;
unsigned int nadd, toint;

 Initialize();

 pulses = 0;

 //pour la barre d'avancement en bas
 chrwait = '.';

 beepon = 1;

 //buffer vide
 plein = 0;
 for (n=0; n<LONG_AQU_TIME; n++) top[n]=0;

 //Par defaut aquisition totale 16 sec
 aqutime = MIN_AQU_TIME;

 //Effet sur afficheur
 lcd_printf((far rom char*)"   Geiger PIC  \0");
 lcd_line(LCD_LINE2);
 lcd_printf((far rom char*)"  v1.3  9/2011 \0");
 TempoS(3);

 lcd_clear();
 lcd_printf((far rom char*)"   uSv/hr = \0");
 lcd_line(LCD_LINE2);
 lcd_printf((far rom char*)"  dose recue\0");
 TempoS(4);
 lcd_clear();

 //Pointeur tableau tournant 10sec
 ntop = 0;

 //Boucle principale
 while (1) {
  //Test si Appui touche
  if (!BOUTON) {
    lcd_clear();
    while (!BOUTON) {
      if (aqutime!=MIN_AQU_TIME) {
        lcd_line(LCD_LINE1);
        lcd_printf((far rom char*)"Mesure en 15sec\0");
        TempoS(3); 
        if (BOUTON) {
          plein = 0;      
          aqutime = MIN_AQU_TIME;
          break;
        }
      }
      if (aqutime!=MID_AQU_TIME) {
        lcd_line(LCD_LINE1);
        lcd_printf((far rom char*)"Mesure en 30sec\0");
        TempoS(3);
        if (BOUTON) {
          plein = 0;
          aqutime = MID_AQU_TIME;
          break;
        }
      }
      if (aqutime!=LONG_AQU_TIME) {
        lcd_line(LCD_LINE1);
        lcd_printf((far rom char*)"Mesure en 120sec\0");
        TempoS(3);
        if (BOUTON) {
          plein = 0;      
          aqutime = LONG_AQU_TIME;
          break;
        }
      }
      if (!beepon) {
        lcd_line(LCD_LINE1);
        lcd_printf((far rom char*)" BUZZER ACTIF   \0");
        beepon = 1;
        TempoS(3);
      } else {
        lcd_line(LCD_LINE1);
        lcd_printf((far rom char*)" PAS DE BUZZER  \0");
        beepon = 0;
        TempoS(3);
      }
      if (BOUTON) break;
    }
    lcd_clear();
  }

  //Marche normale calcul + affichages
  lcd_line(LCD_LINE1);
  lcd_print(' ');

  //Calcule les CPM
  if (plein) {
    // le buffer est plein -> compte en reculant
    if (!ntop) n=aqutime-1; else n=ntop-1;
    for (nadd=0, tot=0; nadd<aqutime; nadd++) {
      tot += top[n];
      if (!n) n=aqutime-1; 
      else n--;
    }
    cpm = tot;
    cpm = (cpm*60) / aqutime;
    toint = cpm;
    Inttochar(toint);
  }
  else {
    //Si pas encore 1 sec 'attente'
    if (ntop) {
      //Buffer pas encore plein
      //Compte ntop mesures en reculant
      for (tot=0, nadd=0, n=ntop-1; nadd<ntop; nadd++) {
        tot += top[n];
        if (!n) n=aqutime-1;
        else n--;
      }
      cpm = (tot * 60) / ntop;
      toint = cpm;
      Inttochar(toint);
    }
  }
   
  //Unite cpm
  lcd_printf((far rom char*)" CPM \0");	// Affiche le 'CPM'
  //Affiche aqutime
  Inttochar(aqutime);
  lcd_printf((far rom char*)"sec \0"); 	  // Affiche le 'sec)'

  //Affichage tournant sur ligne 2
  lcd_line(LCD_LINE2);
  if (affich & 128) {
    //Calcule la dose uSv ou nSv
    //CPM to uSv conversion factor used is 0.0057
    nsv = cpm * 5.7;
    usv = nsv / 1000;

    if ( nsv<1000 ) {
      toint = nsv;
      lcd_print(' ');
      Inttochar(toint);
      lcd_printf((far rom char*)" nSv/hr  \0");
    } else {
      toint = usv;
      lcd_print(' ');
      Inttochar(toint);
      lcd_print(',');
      usv -= toint;
      toint = (usv * 10);
      Inttochar(toint);
      lcd_printf((far rom char*)" uSv/hr  \0");
    }
  }
  else {
    //Calcule la dose de mSv/an
    ur = nsv / 10;
    ur = ur * 8.76;
    toint = ur;
    lcd_print(' ');
    Inttochar(toint);
    lcd_print(',');
    ur -= toint;
    toint = (ur * 10);
    Inttochar(toint);
    lcd_printf((far rom char*)" mSv/an  \0");
 }

 }//fin while(1)
}
/** EOF main.c **************************************************/
