/**************************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 **************************************************************************
 * FileName:        user.c
 * Processor:       PIC18
 *************************************************************************/

/** I N C L U D E S *******************************************************/
#include "p18f2455.h"
#include "iocfg.h"
#include "interrupt.h"
#include "lcd.h"

extern unsigned int pulses;
extern unsigned char tim50, plein;
extern unsigned char top[LONG_AQU_TIME];
extern unsigned char ntop, affich, aqutime, chrwait, beepon;

/************************************************************************
 * Procedure de service d'interruption
 ***********************************************************************/
#pragma code
#pragma interrupt isr
void isr(void) {
 
 //==================================================================
 // Interruption INT0
 //==================================================================
 if (INTCONbits.INT0IF) {
   if (beepon)  BUZZER = 1;		// allume le Buzzer

   //Inc pulses
   pulses++;

   //Lance Timer0 pour 2 mSec
   TMR0L =0;
   //Lance timer 0
   T0CON = 0b11000101;		// prescal/64
   INTCONbits.TMR0IF=0;

   //Reset l'it INT0
   INTCONbits.INT0IF=0;
 }

 //==================================================================
 // Interruption TIMER0
 //==================================================================
 if (INTCONbits.TMR0IF) {
   BUZZER = 0;		// Stoppe le Buzzer
   T0CON = 0;		// Stoppe timer 0
   INTCONbits.TMR0IF=0;
 }

 //==================================================================
 // Interruption TIMER1 -> 50mSec
 //==================================================================
 if (PIR1bits.TMR1IF) {

   TMR1H = 0x6E;
   TMR1L = 0;
   PIR1bits.TMR1IF = 0;

   //Pour affichage tournant (mod 64 pour 3,2 sec)
   affich++;

   //Compteur pour 1 seconde
   tim50--;
   if (!tim50) {
     tim50 = 20;
     //sauve le n top de 1 sec
     top[ntop] = pulses;
     ntop++;
     pulses = 0;

     //historique max sur une 20 ou 120 sec
     if (ntop>aqutime-1) {
       ntop = 0;
       plein = 1;		// signale buffer plein
     }
     //Voit si changer chr barre avancement
     if (!ntop)
       chrwait = (chrwait=='.')?' ':'.';
   }
 }
}

#pragma interrupt isr
void lowisr(void) {
}
#pragma
//===============================================================
